package jp.gr.java_conf.davinche.notepad.widget;

import jp.gr.java_conf.davinche.notepad.R;
import jp.gr.java_conf.davinche.notepad.database.NotepadDto;
import jp.gr.java_conf.davinche.notepad.database.NotepadSqLiteHelper;
import android.app.Activity;
import android.app.PendingIntent;
import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.widget.RemoteViews;

public class TagWidgetProvider extends AppWidgetProvider {

  private static final String sharePreferenceName = "NotepadPreference";

  private static final String sharePropertyId = "notepad_id";

  @Override
  public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
    super.onUpdate(context, appWidgetManager, appWidgetIds);
    for (int mAppWidgetId : appWidgetIds) {
      String id = sharePropertyId + "_" + mAppWidgetId;
      SharedPreferences preferences = context.getSharedPreferences(sharePreferenceName, Activity.MODE_PRIVATE);
      String contentId = preferences.getString(id, "");

      if (contentId != null && !contentId.equals("")) {
        // DBf[^擾
        NotepadSqLiteHelper helper = new NotepadSqLiteHelper(context);
        SQLiteDatabase db = helper.getWritableDatabase();

        String[] columns = {
            "id", "title", "content"
        };
        Cursor cursor = db.query("notepad", columns, "id = " + contentId, null, null, null, "id");
        NotepadDto dto = new NotepadDto();
        while (cursor.moveToNext()) {
          dto.setId(cursor.getString(0));
          dto.setTitle(cursor.getString(1));
          dto.setContent(cursor.getString(2));
        }
        db.close();

        selectContent(context, appWidgetManager, mAppWidgetId, dto);
      }
    }
  }

  @Override
  public void onDeleted(Context context, int[] appWidgetIds) {
    for (int mAppWidgetId : appWidgetIds) {
      String id = sharePropertyId + "_" + mAppWidgetId;
      SharedPreferences preferences = context.getSharedPreferences(sharePreferenceName, Activity.MODE_PRIVATE);
      Editor editor = preferences.edit();
      editor.remove(id);
      editor.commit();
    }
  }

  @Override
  public void onReceive(Context context, Intent intent) {
    super.onReceive(context, intent);
    String contentId = intent.getExtras().getString(OpenNotepadAction.NOTEPAD_CONTENT_ID);

    if (contentId != null && !contentId.equals("")) {
      // DBf[^擾
      NotepadSqLiteHelper helper = new NotepadSqLiteHelper(context);
      SQLiteDatabase db = helper.getWritableDatabase();

      String[] columns = {
          "id", "title", "content"
      };
      Cursor cursor = db.query("notepad", columns, "id = " + contentId, null, null, null, "id");
      NotepadDto dto = new NotepadDto();
      while (cursor.moveToNext()) {
        dto.setId(cursor.getString(0));
        dto.setTitle(cursor.getString(1));
        dto.setContent(cursor.getString(2));
      }
      db.close();

      RemoteViews views = new RemoteViews(context.getPackageName(), R.layout.widget);
      views.setTextViewText(R.id.content, dto.getContent());
      ComponentName thisWidget = new ComponentName(context, TagWidgetProvider.class);
      AppWidgetManager manager = AppWidgetManager.getInstance(context);
      for (int appWidgetId : manager.getAppWidgetIds(thisWidget)) {
        String id = sharePropertyId + "_" + appWidgetId;
        SharedPreferences preferences = context.getSharedPreferences(sharePreferenceName, Activity.MODE_PRIVATE);
        String appContentId = preferences.getString(id, "");
        if (appContentId.equals(contentId)) {
          manager.updateAppWidget(appWidgetId, views);
        }
      }
    }
  }

  public static void selectContent(Context context, AppWidgetManager manager, int appWidgetId, NotepadDto item) {
    RemoteViews views = new RemoteViews(context.getPackageName(), R.layout.widget);
    views.setTextViewText(R.id.content, item.getContent());

    // NbNActionݒ
    Intent openNotepadIntent = new Intent();
    openNotepadIntent.setAction(OpenNotepadAction.ACTION_OPEN_NOTEPAD);
    // \pCegContentIDݒ
    openNotepadIntent.putExtra(OpenNotepadAction.NOTEPAD_CONTENT_ID, item.getId());

    PendingIntent pendingIntent = PendingIntent.getService(context, appWidgetId, openNotepadIntent,
        PendingIntent.FLAG_UPDATE_CURRENT);
    views.setOnClickPendingIntent(R.id.content, pendingIntent);

    manager.updateAppWidget(appWidgetId, views);

    // SharePreferenceAPPIDL[ɂăf[^ێ
    String id = sharePropertyId + "_" + appWidgetId;
    SharedPreferences preferences = context.getSharedPreferences(sharePreferenceName, Activity.MODE_PRIVATE);
    Editor editor = preferences.edit();
    editor.putString(id, item.getId());
    editor.commit();
  }
}
